# This module is part of the Nevow project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

"""Compatibility wrapper over new twisted.python.components,
so that nevow works with it. This module shadows nevow.compy
when using the new Twisted's Zope components system.
"""

from twisted.python.components import *
from twisted.python.reflect import namedAny as _namedAny

_registerAdapter = registerAdapter
def registerAdapter(adapterFactory, origInterface, *interfaceClasses):
    class RandomClazz(object):
        pass
    def namedAny(name):
        if name == '__builtin__.function':
            name='types.FunctionType'
        elif name == '__builtin__.method':
            return RandomClazz # Hack
        elif name == '__builtin__.instancemethod':
            name='types.MethodType'
        elif name == '__builtin__.NoneType':
            name='types.NoneType'
        elif name == '__builtin__.generator':
            name='types.GeneratorType'
        return _namedAny(name)

    isStr = type(adapterFactory) is str
    if (type(origInterface) is str) != isStr:
        raise ValueError("Either all arguments must be strings or all must be objects.")
    
    for interfaceClass in interfaceClasses:
        if (type(interfaceClass) is str) != isStr:
            raise ValueError("Either all arguments must be strings or all must be objects.")

    if isStr:
#        print "registerAdapter:",adapterFactory, origInterface, interfaceClasses
        adapterFactory = namedAny(adapterFactory)
        origInterface = namedAny(origInterface)
        interfaceClasses = [namedAny(x) for x in interfaceClasses]
#        print "_registerAdapter:",adapterFactory, origInterface, interfaceClasses
    _registerAdapter(adapterFactory, origInterface, *interfaceClasses)


class IComponentized(Interface):
    pass

_Componentized = Componentized
class Componentized(_Componentized):
    __implements__ = (IComponentized,)
    
    def __init__(self, adapterCache=None):
        _Componentized.__init__(self)
        if adapterCache:
            for k, v in adapterCache.items():
                self.setComponent(k, v)

