

## liveevil

from twisted.internet import defer
from twisted.internet import reactor

from nevow import events, tags
from nevow import inevow
from nevow import compy
from nevow import testutil
from nevow import context
from nevow.flat import flatten


def callJS(func, *args):
    """Return a Javascript string formatted such that the JS function named by the string 'func'
    will be called and the args specified as strings in *args will be quoted properly to prevent JS
    syntax errors.
    """
    args = ["'%s'" % a.replace("'", "\\'").replace('"', '\\"') for a in args]
    return "%s(%s);" % (func, ','.join(args))


class ILiveEvil(compy.Interface):
    def hookupOutput(self, output, finisher=None):
        """hook up an output conduit to this live evil instance.
        """

    def sendScript(self, script):
        """send a script through the output conduit to the browser.
        If no output conduit is yet hooked up, buffer the script
        until one is.
        """

    def handleInput(self, identifier, *args):
        """route some input from the browser to the appropriate
        destination.
        """


class literal(str):
    pass


class LiveEvil(object):
    """An object which represents the client-side webbrowser.
    """
    __implements__ = ILiveEvil,

    output = None
    finisher = None
    def __init__(self, request, credentials):
        request.getSession().setComponent(ILiveEvil, self)
        # TODO set up an event hub
        self.events = events.EventNotification()
        self.hookupNotifications = []
        self.outputBuffer = []

    def hookupOutput(self, page, output, finisher = None):
        print "output hooked up.", page
        self.output = output
        self.finisher = finisher
        for buf in self.outputBuffer:
            self.sendScript(buf)
            #print "Sent buffered item: ", buf
        self.outputBuffer = []
        for notify in self.hookupNotifications:
            notify(self)

    def addNotification(self, notify):
        self.hookupNotifications.append(notify)

    def sendScript(self, script):
        if self.output:
            #print "output sent!", script
            prefix = '%8i' % len(script)
            if len(prefix) > 8:
                raise RuntimeError("Tried to send too much script to the browser at once; sorry.")
            self.output('%s%s' % (prefix, script))
        else:
            self.outputBuffer.append(script)
            print "output buffered!", script

    def handleInput(self, identifier, *args):
        #self.sendScript(input)
        self.events.publish(identifier, *(self, ) + args)

    ## Here is some api your handlers can use to more easily manipulate the live page
    def flt(client, stan, quote=True):
        """Flatten some stan to a string suitable for embedding in a javascript string.
        
        If quote is True, apostrophe, quote, and newline will be quoted ('"\n)
        """
        fr = testutil.FakeRequest()
        fr.getSession().setComponent(ILiveEvil, client)
        ctx = context.RequestContext(tag=fr).with(context.PageContext())
        fl = flatten(stan, ctx=ctx)
        if quote:
            fl = fl.replace("'", "\\'").replace('"', '\\"').replace('\n', '\\n')
        return fl

    def set(client, what, to):
        """Set the node with the id 'what' to the stan 'to'
        """
        client.sendScript("nevow_setNode('%s', '%s');" % (client.flt(what), client.flt(to)))

    def alert(client, what):
        """Show the user an alert 'what'
        """
        client.sendScript("alert('%s');" % (client.flt(what), ))

    def append(client, where, what):
        """Append the stan 'what' to the node with the id 'where'
        """
        client.sendScript(
            "nevow_appendNode('%s', '%s');" % (client.flt(where), client.flt(what)))


class Output(object):
    __implements__ = inevow.IResource,

    def renderHTTP(self, request):
        session = request.getSession()
        self.mind = mind = session.getComponent(ILiveEvil)

        ## When the connection is lost, reset LiveEvil.output to None
        ## so we know when to buffer and when not to buffer
        def _resetOutput(results, liveevil):
            print "CONNECTION WAS LOST!!!"
            liveevil.output = None
            return results
        request.notifyFinish().addBoth(_resetOutput, mind)

        mind.hookupOutput(request.resources[-2], request.write, request.finish)
        reactor.callLater(30, self.noop, session)
        ## This deferred never fires!@ haha
        return defer.Deferred()

    def noop(self, session):
        if self.mind.output:
            session.touch()
            self.mind.sendScript('null;')
            reactor.callLater(30, self.noop, session)


liveOutput = Output()


class Input(object):
    __implements__ = inevow.IResource,

    def renderHTTP(self, request):
        mind = request.getSession().getComponent(ILiveEvil)
        reactor.callLater(0, mind.handleInput, request.args['target'][0], *request.args.get('arguments',()))
        return "<html><body>done</body></html>"


liveInput = Input()


glue = tags.script(language="javascript")[
    tags.xml("""
offset = 0;
xmlhttp = new XMLHttpRequest();
var currentState = 'new';
var currentScriptSize = 0;
var currentScript = '';

xmlhttp.onreadystatechange = function() {
  if (xmlhttp.readyState == 3) {
    var content = xmlhttp.responseText.substring(offset, xmlhttp.responseText.length);
    offset += content.length;
    dump("*** " + content.length + " Content received for state " + currentState + '\\n');
    while(content) {
        if (currentState == 'new') {
            currentScriptSize = parseInt(content.substring(0, 8));
            content = content.substring(8, content.length);
            if (currentScriptSize < content.length) {
                currentScript = content.substring(0, currentScriptSize);
                content = content.substring(currentScriptSize, content.length);
            } else {
                currentScript = content;
                content = '';
            }
            currentScriptSize -= currentScript.length;
            currentState = 'continuation';
        } else {
            if (currentScriptSize < content.length) {
                currentScript += content.substring(0, currentScriptSize);
                content = content.substring(currentScriptSize, content.length);
                currentScriptSize = 0;
            } else {
                currentScript += content;
                currentScriptSize -= content.length;
                content = '';
            }
        }
        if (currentScriptSize == 0) {
            dump("eval currentScript:\\n");
            dump(currentScript+'\\n');
            dump("END eval\\n");
            currentState = 'new';
            eval(currentScript);
        }
    }
  } else if (xmlhttp.readyState == 4) {
        //alert(xmlhttp.responseText)
  }
}

xmlhttp.open("GET", "nevow_liveOutput", true)
xmlhttp.send(null)

function nevow_clientToServerEvent(theTarget, node) {
    var additionalArguments = ''
    for (i = 2; i<arguments.length; i++) {
        additionalArguments += '&arguments='
        additionalArguments += escape(eval(arguments[i]))
    }

    input = new XMLHttpRequest();
    input.onreadystatechange = function() {
        //alert('change');
    }
    input.open("GET", "nevow_liveInput?target=" + escape(theTarget)+additionalArguments)
    input.send(null)
}

function nevow_setNode(node, to) {
    document.getElementById(node).innerHTML = to;
}

function nevow_appendNode(node, what) {
    var oldnode = document.getElementById(node);
    var newspan = document.createElement('span');
    newspan.innerHTML = what;
    for (i=0; i<newspan.childNodes.length; i++) {
        oldnode.appendChild(newspan.childNodes[i]);
    }
}
""")]


i = 0
def next():
    global i
    i += 1
    return i


def handler(callable, *args):
    def block(context, data):
        identifier = "%x" % next()
        try:
            mind = context.locate(ILiveEvil)
        except KeyError:
            # Remove this when the request context looks in components and the session context is in place and also looks in components.
            mind = context.locate(inevow.IRequest).getSession().getComponent(ILiveEvil)
        mind.events.subscribe(identifier, callable)
        argstr = ','.join([isinstance(x, literal) and str(x) or "'%s'" % x.replace("'", "\\'") for x in args])
        if argstr:
            argstr = ','+argstr
        js = "nevow_clientToServerEvent('%(id)s', this%(argstr)s); return false;" % {'id': identifier, 'argstr': argstr}
        #print "JS", js
        return js
    return block

